package com.example.spiritlevel;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.drawable.Drawable;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.drawable.LayerDrawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.OvalShape;
import android.hardware.SensorManager;
import android.view.View;
import android.util.Log;

public class SpiritLevelDrawableView extends View {
  private LayerDrawable layer;
  private ShapeDrawable bubble;

  private static final String TAG = "SpiritlevelDrawableView";

  private static final double WIDTH_MAGNIFY = 1.5;
  private static final double HEIGHT_MAGNIFY = 3;

  private int bubbleOrigX;
  private int bubbleOrigY;
  private int bubbleX;
  private int bubbleY;
  private int bubbleDiam = 20;

  public SpiritLevelDrawableView(Context context) {
    super(context);
    
    Resources res = getResources();

    int outerX = 80;
    int outerY = 50;
    int outerWidth = 150;
    int outerHeight = 300;

    int liquidWidth = (int) ((SensorManager.STANDARD_GRAVITY 
        * WIDTH_MAGNIFY * 2) + bubbleDiam + 0.5);
    int liquidHeight = (int) ((SensorManager.STANDARD_GRAVITY 
        * HEIGHT_MAGNIFY * 2) + bubbleDiam + 0.5);
    int liquidX = outerX + (outerWidth-liquidWidth)/2;
    int liquidY = outerY + (outerHeight-liquidHeight)/2;

    bubbleX = bubbleOrigX = outerX + (outerWidth/2) - (bubbleDiam/2);
    bubbleY = bubbleOrigY = outerY + (outerHeight/2) - (bubbleDiam/2);
    //Log.i(TAG, "Drawing: bubbleY = " + bubbleY);

    ShapeDrawable outer = new ShapeDrawable();
    outer.getPaint().setColor(res.getColor(R.color.brightpurple));
    outer.setBounds(outerX, outerY, outerX + outerWidth, 
        outerY + outerHeight);

    ShapeDrawable liquid = new ShapeDrawable();
    liquid.getPaint().setColor(res.getColor(R.color.brightyellow));
    liquid.setBounds(liquidX, liquidY, liquidX + liquidWidth, 
        liquidY + liquidHeight);

    bubble = new ShapeDrawable(new OvalShape());
    bubble.getPaint().setColor(Color.BLACK);
    bubble.setBounds(bubbleX, bubbleY, bubbleX + bubbleDiam, 
        bubbleY + bubbleDiam);

    layer = new LayerDrawable(new Drawable[] {outer, liquid, bubble});
  }

  protected void onDraw(Canvas canvas) {
    bubble.setBounds(bubbleX, bubbleY, bubbleX + bubbleDiam, 
        bubbleY + bubbleDiam);
    layer.draw(canvas);
  }

  protected void moveBubble(float gravX, float gravY) {
    bubbleX = bubbleOrigX + (int) (gravX * WIDTH_MAGNIFY);
    bubbleY = bubbleOrigY - (int) (gravY * HEIGHT_MAGNIFY);
  }
}
