/* 
 * Code to drive a 24c64 EEPROM and DS1307 connected via Wire library
 */

#include <Wire.h>      //I2C library
int DEVICETYPE = 0x50; //24c64 eeprom default address value
int DS1307 = 0x68;
 
int i;
//int delta = 1000*60*5;
int delta =300;

byte second, minute, hour, dayOfWeek, day, month, year;
byte test; 
int temp;
byte data[]={ 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF     };
int blockcount = 0;
///Application variables
int BUTTON =9;
int sensorPin = A0; 
int sensorData = 0;
int LED =13;
int eprom1 = DEVICETYPE; //I2C address of EEPROM
boolean rc =true; //used for return codes
unsigned int addr =0x00; //address counter


// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
} 

boolean eeprom_write_rand( int device, unsigned int location, byte data ) {
  Wire.beginTransmission(device);
  int MSB = location >> 8;
  int LSB = location & 0xFF;
  Wire.send(MSB); 
  Wire.send(LSB); 
  Wire.send(data);
  Wire.endTransmission();
  delay(10);
}

boolean eeprom_write_page( int device, unsigned int location, byte* buffer, int length) {
  //NB - wire library is currently hardcoded to a buffer length of 30 bytes. I recompiled mine)
  Wire.beginTransmission(device);
  int MSB = location >> 8;
  int LSB = location & 0xFF;   
  /* 
   For 24c64 EEPROM
   xxx1111111100000 = page address = 0x0f 0xe0
   xxx0000000011111 = byte address = 0x1f
   */
  Wire.send(MSB); 
  Wire.send(LSB); 
  int byteaddress = LSB & 0x1F;
  if (byteaddress + length > 32) {
    return false; // no writing over page boundary
  }
  int i;
  for ( i = 0; i < length; i++ ){
    Wire.send(buffer[i]); // send bytes
  }
  Wire.endTransmission();
  return true;

}


byte eeprom_read_rand( int device, unsigned int location ) {
  byte data = 0xFF;
  Wire.beginTransmission(device);
  int MSB = location >> 8;
  int LSB = location & 0xFF;
  Wire.send(MSB); 
  Wire.send(LSB); 
  Wire.endTransmission();
  Wire.requestFrom(device,1);
  if (Wire.available())
    data = Wire.receive();
  return data;
}

boolean eeprom_read_seq( int device, unsigned int location, int length, byte *readdata) {
  // check for valid input
  if (length > 32) 
    return false; //longest sequence possible
  Wire.beginTransmission(device);
  int MSB = location >> 8;
  int LSB = location & 0xFF;
  Wire.send(MSB);   // fake write to set the address
  Wire.send(LSB); 
  Wire.endTransmission();
  Wire.requestFrom(device,length); // now read back 
  int i;
  for ( i = 0; i < length; i++ ){
    if (Wire.available()) readdata[i] = Wire.receive();
    //delay(10);
  }
  return true;
}

void dumpData() 
{
  int eprom1 = DEVICETYPE; //I2C address of EEPROM
  boolean rc =true; //used for return codes
  unsigned int location =0; //first address
  byte buffer[32]; // a buffer full of bytes
  byte b1 = 0xFF; // access the first address from the memory
  int chunk = 0;

  while (location!=256) 
  {
    rc = eeprom_read_seq(eprom1,location*32,32,(byte *)buffer);
    //Serial.println("dumping byte data as hex");
    int i;
    //for ( i = 0; i < sizeof(buffer); i++ ){
    //  Serial.print(buffer[i], BYTE);
    //  Serial.print("  ");
    //}  
    while (chunk<4){
      Serial.println(" ");
      for ( i = chunk*8; i < (chunk+1)*8; i++ ){
        Serial.print(buffer[i], DEC);
        Serial.print(", ");
      }  
      Serial.print(0x0A);
      chunk++;
    }
    chunk=0;
    location++; //increase address
  }
  Serial.println(" ");
  Serial.println("ENDS");

}

int convertAnalogueToTemp(int reading) {
  float mvperunit = 4.9;
  return(((int)((reading*mvperunit)-400)/.195));
}

void printTime () {
  Serial.print (day, DEC);
  Serial.print ("/");
  Serial.print (month, DEC);
  Serial.print ("/");
  Serial.print (year, DEC);
  Serial.print ("-");
  Serial.print (hour, DEC);
  Serial.print (":");
  Serial.print (minute, DEC);
  Serial.print (":");
  Serial.print (second, DEC);
}

void getTime() {
  // Reset the register pointer
  Wire.beginTransmission(DS1307);
  Wire.send(0x00);
  Wire.endTransmission();
  Wire.requestFrom(DS1307, 7);
  // A few of these need masks because certain bits are control bits
  second     = bcdToDec(Wire.receive() & 0x7f);
  minute     = bcdToDec(Wire.receive());
  hour       = bcdToDec(Wire.receive() & 0x3f);  // Need to change this for am/pm
  dayOfWeek  = bcdToDec(Wire.receive());
  day        = bcdToDec(Wire.receive());
  month      = bcdToDec(Wire.receive());
  year       = bcdToDec(Wire.receive());
}

void setup() 
{
  pinMode(sensorPin, INPUT);
  pinMode(BUTTON, INPUT);
  pinMode(LED, OUTPUT);
  Wire.begin(); // initialise the connection
  Serial.begin(57600);
  //set the time
  second = 00;
  minute = 11;
  hour   = 02;
  dayOfWeek = 6;
  day    = 30;
  month  = 4;
  year   = 11;

  Wire.beginTransmission(DS1307);
  Wire.send(0x00);
  Wire.send(decToBcd(second));    // 0 to bit 7 starts the clock
  Wire.send(decToBcd(minute));
  Wire.send(decToBcd(hour));      // If you want 12 hour am/pm you need to set
  // bit 6 (also need to change readDateDs1307)
  Wire.send(decToBcd(dayOfWeek));
  Wire.send(decToBcd(day));
  Wire.send(decToBcd(month));
  Wire.send(decToBcd(year));
  Wire.endTransmission();
}

void loop() 
{ 
  int val = 0;
  int offset =0;
  int i;
  blockcount =0;
  while (blockcount<4){ 
    ///read temperature data
    sensorData = analogRead(sensorPin);
    temp =convertAnalogueToTemp(sensorData);

    //dump to serial
    Serial.print(sensorData,DEC);
    Serial.print(" = ");
    Serial.print(temp);
    Serial.print("C");
    Serial.println("  ");

    ///pause for next reading
    for ( i = 0; i < delta; i++ ){
      delay(650);
      digitalWrite(LED, HIGH);  
      delay(200);                 
      digitalWrite(LED, LOW); 
      // we should check if the 'dump' key is pressed

      val = digitalRead(BUTTON);
      if (val == HIGH) dumpData();
    }  

    getTime();
    printTime();
    Serial.println();
    ///package up bytes;
    offset = 8 * blockcount;
    data[offset]  = (byte) day;
    data[offset+1]= (byte) month;
    data[offset+2]= (byte) year;
    data[offset+3]= (byte) hour;
    data[offset+4]= (byte) minute;
    data[offset+5]= (byte) second;
    data[offset+6]= (byte) (temp >> 8); //msb of temp
    data[offset+7]= (byte) (temp & 0xFF); 
    blockcount++;
  }

  rc = eeprom_write_page( eprom1, addr*32, (byte *)data, 32);
  for ( i = 0; i < sizeof(data); i++ ){
    Serial.print(data[i], DEC);
    Serial.print("  ");
  }  
  Serial.println("  ");
  rc = eeprom_read_seq( eprom1, addr*32, 32, (byte *)data);

  for ( i = 0; i < sizeof(data); i++ ){
    Serial.print(data[i], DEC);
    Serial.print("  ");
  }  
  if (rc) {
    digitalWrite(LED, HIGH);  
    delay(10);                 
    digitalWrite(LED, LOW);
  }
  addr++;
  if (addr > 255) addr =0;
}








